@@ -4,19 +4,16 @@
 #include "compileroptionsbuilder_test.h"
 
 #include "compileroptionsbuilder.h"
-#include "projectinfo.h"
 #include "projectpart.h"
 
 #include <projectexplorer/headerpath.h>
-#include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <utils/algorithm.h>
+#include <utils/qtcassert.h>
 #include <utils/temporarydirectory.h>
 
 #include <QtTest>
 
-#include <memory>
-
 using namespace ProjectExplorer;
 
 namespace CppEditor::Internal {
@@ -28,14 +25,14 @@ public:
     const ProjectPart &finalize()
     {
         QFile pchFile(pchFileNativePath());
-        pchFile.open(QIODevice::WriteOnly);
+        QTC_CHECK(pchFile.open(QIODevice::WriteOnly));
         RawProjectPart rpp;
         rpp.setPreCompiledHeaders({pchFileNativePath()});
         rpp.setMacros({Macro{"projectFoo", "projectBar"}});
         rpp.setQtVersion(Utils::QtMajorVersion::Qt5);
         rpp.setHeaderPaths(headerPaths);
         rpp.setConfigFileName(projectConfigFile);
-        ToolChainInfo tcInfo;
+        ToolchainInfo tcInfo;
         tcInfo.type = toolchainType;
         tcInfo.targetTriple = targetTriple;
         tcInfo.abi = Abi::fromString(targetTriple);
@@ -43,10 +40,10 @@ public:
             tcInfo.abi = Abi(Abi::X86Architecture, Abi::DarwinOS, Abi::FreeBsdFlavor,
                              Abi::MachOFormat, 64);
         }
-        tcInfo.isMsvc2015ToolChain = isMsvc2015;
+        tcInfo.isMsvc2015Toolchain = isMsvc2015;
         tcInfo.extraCodeModelFlags = extraFlags;
         tcInfo.macroInspectionRunner = [this](const QStringList &) {
-            return ToolChain::MacroInspectionReport{toolchainMacros, languageVersion};
+            return Toolchain::MacroInspectionReport{toolchainMacros, languageVersion};
         };
         RawProjectPartFlags rppFlags;
         rppFlags.commandLineFlags = flags;
@@ -196,7 +193,7 @@ void CompilerOptionsBuilderTest::testHeaderPathOptionsOrder()
 
     QCOMPARE(compilerOptionsBuilder.options(),
              (QStringList{"-nostdinc", "-nostdinc++", "-I" + t.toNative("/tmp/path"),
-                         "-I" + t.toNative("/tmp/system_path"), "-isystem", t.toNative("/dummy"),
+                          "-isystem", t.toNative("/tmp/system_path"), "-isystem", t.toNative("/dummy"),
                           "-isystem", t.toNative("/tmp/builtin_path")}));
 }
 
@@ -212,9 +209,9 @@ void CompilerOptionsBuilderTest::testHeaderPathOptionsOrderMsvc()
 
     QCOMPARE(compilerOptionsBuilder.options(),
              (QStringList{"-nostdinc", "-nostdinc++", "-I" + t.toNative("/tmp/path"),
-                            "-I" + t.toNative("/tmp/system_path"), "/clang:-isystem",
-                          "/clang:" + t.toNative("/dummy"), "/clang:-isystem",
-                          "/clang:" + t.toNative("/tmp/builtin_path")}));
+                          "/clang:-isystem", "/clang:" + t.toNative("/tmp/system_path"),
+                          "/clang:-isystem", "/clang:" + t.toNative("/dummy"),
+                          "/clang:-isystem", "/clang:" + t.toNative("/tmp/builtin_path")}));
 }
 
 void CompilerOptionsBuilderTest::testUseSystemHeader()
@@ -239,7 +236,7 @@ void CompilerOptionsBuilderTest::testNoClangHeadersPath()
     t.compilerOptionsBuilder->addHeaderPathOptions();
 
     QCOMPARE(t.compilerOptionsBuilder->options(),
-             (QStringList{"-I" + t.toNative("/tmp/path"), "-I" + t.toNative("/tmp/system_path")}));
+             (QStringList{"-I" + t.toNative("/tmp/path"), "-isystem", t.toNative("/tmp/system_path")}));
 }
 
 void CompilerOptionsBuilderTest::testClangHeadersAndCppIncludePathsOrderMacOs()
@@ -260,7 +257,7 @@ void CompilerOptionsBuilderTest::testClangHeadersAndCppIncludePathsOrderMacOs()
 
     QCOMPARE(compilerOptionsBuilder.options(),
              (QStringList{"-nostdinc", "-nostdinc++", "-I" + t.toNative("/tmp/path"),
-                          "-I" + t.toNative("/tmp/system_path"),
+                          "-isystem", t.toNative("/tmp/system_path"),
                           "-isystem", t.toNative("/usr/include/c++/4.2.1"),
                           "-isystem", t.toNative("/usr/include/c++/4.2.1/backward"),
                           "-isystem", t.toNative("/usr/local/include"),
@@ -599,7 +596,7 @@ void CompilerOptionsBuilderTest::testBuildAllOptions()
                           wrappedQtHeadersPath,         // contains -I already
                           wrappedQtCoreHeadersPath,     // contains -I already
                           "-I" + t.toNative("/tmp/path"),
-                          "-I" + t.toNative("/tmp/system_path"),
+                          "-isystem", t.toNative("/tmp/system_path"),
                           "-isystem", t.toNative("/dummy"),
                           "-isystem", t.toNative("/tmp/builtin_path")}));
 }
@@ -628,7 +625,7 @@ void CompilerOptionsBuilderTest::testBuildAllOptionsMsvc()
                           wrappedQtHeadersPath,         // contains -I already
                           wrappedQtCoreHeadersPath,     // contains -I already
                           "-I" + t.toNative("/tmp/path"),
-                          "-I" + t.toNative("/tmp/system_path"),
+                          "/clang:-isystem", "/clang:" + t.toNative("/tmp/system_path"),
                           "/clang:-isystem", "/clang:" + t.toNative("/dummy"),
                           "/clang:-isystem", "/clang:" + t.toNative("/tmp/builtin_path")}));
 }
@@ -659,7 +656,7 @@ void CompilerOptionsBuilderTest::testBuildAllOptionsMsvcWithExceptions()
                           wrappedQtHeadersPath,             // contains -I already
                           wrappedQtCoreHeadersPath,         // contains -I already
                           "-I" + t.toNative("/tmp/path"),
-                          "-I" + t.toNative("/tmp/system_path"),
+                          "/clang:-isystem", "/clang:" + t.toNative("/tmp/system_path"),
                           "/clang:-isystem", "/clang:" + t.toNative("/dummy"),
                           "/clang:-isystem", "/clang:" + t.toNative("/tmp/builtin_path")}));
 }
