@@ -1,33 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 Denis Shienkov <denis.shienkov@gmail.com>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2019 Denis Shienkov <denis.shienkov@gmail.com>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "peripheralregisterhandler.h"
 
 #include "debuggeractions.h"
 #include "debuggercore.h"
-#include "debuggerdialogs.h"
+#include "debuggertr.h"
+#include "registerhandler.h"
 
 #include <utils/basetreeview.h>
 
@@ -37,13 +16,12 @@
 #include <QLineEdit>
 #include <QMenu>
 #include <QPainter>
+#include <QXmlStreamReader>
 
 using namespace Utils;
 
-namespace Debugger {
-namespace Internal {
+namespace Debugger::Internal {
 
-namespace {
 // Keys of a properties in SVD file.
 constexpr char kAccess[] = "access";
 constexpr char kAddressOffset[] = "addressOffset";
@@ -70,7 +48,6 @@ constexpr char kRegisters[] = "registers";
 constexpr char kResetvalue[] = "resetValue";
 constexpr char kSize[] = "size";
 constexpr char kWritOnlye[] = "write-only";
-} // namespace
 
 enum PeripheralRegisterColumns
 {
@@ -89,13 +66,13 @@ static QString accessName(PeripheralRegisterAccess access)
 {
     switch (access) {
     case PeripheralRegisterAccess::ReadOnly:
-        return PeripheralRegisterHandler::tr("RO");
+        return Tr::tr("RO");
     case PeripheralRegisterAccess::WriteOnly:
-        return PeripheralRegisterHandler::tr("WO");
+        return Tr::tr("WO");
     case PeripheralRegisterAccess::ReadWrite:
-        return PeripheralRegisterHandler::tr("RW");
+        return Tr::tr("RW");
     default:
-        return PeripheralRegisterHandler::tr("N/A");
+        return Tr::tr("N/A");
     }
 }
 
@@ -141,22 +118,22 @@ static quint64 valueFromString(const QString &string, PeripheralRegisterFormat f
                                bool *ok)
 {
     if (fmt == PeripheralRegisterFormat::Hexadecimal) {
-        const QRegularExpression re("^(0x)?([0-9A-F]+)$");
+        static const QRegularExpression re("^(0x)?([0-9A-F]+)$");
         const QRegularExpressionMatch m = re.match(string);
         if (m.hasMatch())
             return m.captured(2).toULongLong(ok, 16);
     } else if (fmt == PeripheralRegisterFormat::Decimal) {
-        const QRegularExpression re("^([0-9]+)$");
+        static const QRegularExpression re("^([0-9]+)$");
         const QRegularExpressionMatch m = re.match(string);
         if (m.hasMatch())
             return m.captured(1).toULongLong(ok, 10);
     } else if (fmt == PeripheralRegisterFormat::Octal) {
-        const QRegularExpression re("^(0)?([0-7]+)$");
+        static const QRegularExpression re("^(0)?([0-7]+)$");
         const QRegularExpressionMatch m = re.match(string);
         if (m.hasMatch())
             return m.captured(2).toULongLong(ok, 8);
     } else if (fmt == PeripheralRegisterFormat::Binary) {
-        const QRegularExpression re("^(0b)?([0-1]+)$");
+        static const QRegularExpression re("^(0b)?([0-1]+)$");
         const QRegularExpressionMatch m = re.match(string);
         if (m.hasMatch())
             return m.captured(2).toULongLong(ok, 2);
@@ -172,7 +149,7 @@ QString PeripheralRegisterField::bitRangeString() const
 {
     const int from = bitOffset;
     const int to = bitOffset + bitWidth - 1;
-    return PeripheralRegisterHandler::tr("[%1..%2]").arg(to).arg(from);
+    return Tr::tr("[%1..%2]").arg(to).arg(from);
 }
 
 QString PeripheralRegisterField::bitValueString(quint64 regValue) const
@@ -472,103 +449,20 @@ void PeripheralRegisterItem::triggerChange()
                                          m_reg.currentValue.v);
 }
 
-// PeripheralRegisterDelegate
-
-class PeripheralRegisterDelegate final : public QItemDelegate
-{
-public:
-    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &,
-        const QModelIndex &index) const final
-    {
-        if (index.column() == PeripheralRegisterValueColumn) {
-            const auto lineEdit = new QLineEdit(parent);
-            lineEdit->setAlignment(Qt::AlignLeft);
-            lineEdit->setFrame(false);
-            return lineEdit;
-        }
-        return nullptr;
-    }
-
-    void setEditorData(QWidget *editor, const QModelIndex &index) const final
-    {
-        const auto lineEdit = qobject_cast<QLineEdit *>(editor);
-        QTC_ASSERT(lineEdit, return);
-        lineEdit->setText(index.data(Qt::EditRole).toString());
-    }
-
-    void setModelData(QWidget *editor, QAbstractItemModel *model,
-        const QModelIndex &index) const final
-    {
-        if (index.column() == PeripheralRegisterValueColumn) {
-            const auto lineEdit = qobject_cast<QLineEdit *>(editor);
-            QTC_ASSERT(lineEdit, return);
-            model->setData(index, lineEdit->text(), Qt::EditRole);
-        }
-    }
-
-    void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option,
-        const QModelIndex &) const final
-    {
-        editor->setGeometry(option.rect);
-    }
-
-    void paint(QPainter *painter, const QStyleOptionViewItem &option,
-        const QModelIndex &index) const final
-    {
-        if (index.column() == PeripheralRegisterValueColumn) {
-            const bool paintRed = index.data(PeripheralRegisterChangedRole).toBool();
-            const QPen oldPen = painter->pen();
-            const QColor lightColor(140, 140, 140);
-            if (paintRed)
-                painter->setPen(QColor(200, 0, 0));
-            else
-                painter->setPen(lightColor);
-            // FIXME: performance? this changes only on real font changes.
-            const QFontMetrics fm(option.font);
-            const int charWidth = qMax(fm.horizontalAdvance('x'),
-                                       fm.horizontalAdvance('0'));
-            const QString str = index.data(Qt::DisplayRole).toString();
-            int x = option.rect.x();
-            bool light = !paintRed;
-            for (int i = 0; i < str.size(); ++i) {
-                const QChar c = str.at(i);
-                const int uc = c.unicode();
-                if (light && (uc != 'x' && uc != '0')) {
-                    light = false;
-                    painter->setPen(oldPen.color());
-                }
-                if (uc == ' ') {
-                    light = true;
-                    painter->setPen(lightColor);
-                } else {
-                    QRect r = option.rect;
-                    r.setX(x);
-                    r.setWidth(charWidth);
-                    painter->drawText(r, Qt::AlignHCenter, c);
-                }
-                x += charWidth;
-            }
-            painter->setPen(oldPen);
-        } else {
-            QItemDelegate::paint(painter, option, index);
-        }
-    }
-};
-
 // PeripheralRegisterHandler
 
 PeripheralRegisterHandler::PeripheralRegisterHandler(DebuggerEngine *engine)
     : m_engine(engine)
 {
     setObjectName("PeripheralRegisterModel");
-    setHeader({tr("Name"), tr("Value"), tr("Access")});
+    setHeader({Tr::tr("Name"), Tr::tr("Value"), Tr::tr("Access")});
 }
 
 static void handleField(QXmlStreamReader &in, PeripheralRegister &reg)
 {
     PeripheralRegisterField fld;
-    Utils::optional<int> from;
-    Utils::optional<int> to;
+    std::optional<int> from;
+    std::optional<int> to;
     while (in.readNextStartElement()) {
         const auto elementName = in.name();
         if (elementName == QLatin1String(kName)) {
@@ -698,7 +592,7 @@ static void handleGroup(QXmlStreamReader &in, PeripheralRegisterGroups &groups)
 
 static PeripheralRegisterGroups availablePeripheralRegisterGroups(const FilePath &filePath)
 {
-    QFile f(filePath.toString());
+    QFile f(filePath.toFSPathString());
     if (!f.open(QIODevice::ReadOnly))
         return {};
 
@@ -731,12 +625,9 @@ void PeripheralRegisterHandler::updateRegisterGroups()
     clear();
 
     const DebuggerRunParameters &rp = m_engine->runParameters();
-    const FilePath peripheralDescriptionFile = FilePath::fromVariant(
-                rp.inferior.extraData.value(Debugger::Constants::kPeripheralDescriptionFile));
-
-    if (!peripheralDescriptionFile.exists())
+    if (!rp.peripheralDescriptionFile().exists())
         return;
-    m_peripheralRegisterGroups = availablePeripheralRegisterGroups(peripheralDescriptionFile);
+    m_peripheralRegisterGroups = availablePeripheralRegisterGroups(rp.peripheralDescriptionFile());
 }
 
 void PeripheralRegisterHandler::updateRegister(quint64 address, quint64 value)
@@ -759,8 +650,7 @@ QList<quint64> PeripheralRegisterHandler::activeRegisters() const
 QVariant PeripheralRegisterHandler::data(const QModelIndex &idx, int role) const
 {
     if (role == BaseTreeView::ItemDelegateRole) {
-        return QVariant::fromValue(static_cast<QAbstractItemDelegate *>(
-                                       new PeripheralRegisterDelegate));
+        return QVariant::fromValue(createRegisterDelegate(PeripheralRegisterValueColumn));
     }
     return PeripheralRegisterModel::data(idx, role);
 }
@@ -798,29 +688,31 @@ bool PeripheralRegisterHandler::contextMenuEvent(const ItemViewEvent &ev)
         menu->addMenu(fmtMenu);
     }
 
-    menu->addAction(debuggerSettings()->settingsDialog.action());
+    menu->addAction(settings().settingsDialog.action());
+    connect(menu, &QMenu::aboutToHide, menu, &QObject::deleteLater);
     menu->popup(ev.globalPos());
     return true;
 }
 
-QMenu *PeripheralRegisterHandler::createRegisterGroupsMenu(DebuggerState state) const
+QMenu *PeripheralRegisterHandler::createRegisterGroupsMenu(DebuggerState state)
 {
-    const auto groupMenu = new QMenu(tr("View Groups"));
+    const auto groupMenu = new QMenu(Tr::tr("View Groups"));
     const auto actionGroup = new QActionGroup(groupMenu);
     bool hasActions = false;
-    for (const PeripheralRegisterGroup &group : qAsConst(m_peripheralRegisterGroups)) {
-        const QString actName = QStringLiteral("%1: %2")
-                .arg(group.name, group.description);
+    for (const PeripheralRegisterGroup &group : std::as_const(m_peripheralRegisterGroups)) {
+        const QString groupName = group.name;
+        const QString actName = QStringLiteral("%1: %2").arg(groupName, group.description);
         QAction *act = groupMenu->addAction(actName);
         const bool on = m_engine->hasCapability(RegisterCapability)
                 && (state == InferiorStopOk || state == InferiorUnrunnable);
         act->setEnabled(on);
-        act->setData(group.name);
         act->setCheckable(true);
         act->setChecked(group.active);
         actionGroup->addAction(act);
-        QObject::connect(act, &QAction::triggered,
-                         this, &PeripheralRegisterHandler::setActiveGroup);
+        QObject::connect(act, &QAction::triggered, this, [this, groupName](bool checked) {
+            if (checked)
+                setActiveGroup(groupName);
+        });
         hasActions = true;
     }
     groupMenu->setEnabled(hasActions);
@@ -831,7 +723,7 @@ QMenu *PeripheralRegisterHandler::createRegisterGroupsMenu(DebuggerState state)
 QMenu *PeripheralRegisterHandler::createRegisterFormatMenu(
         DebuggerState state, PeripheralRegisterItem *item) const
 {
-    const auto fmtMenu = new QMenu(tr("Format"));
+    const auto fmtMenu = new QMenu(Tr::tr("Format"));
     const auto actionGroup = new QActionGroup(fmtMenu);
 
     const bool on = m_engine->hasCapability(RegisterCapability)
@@ -841,7 +733,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFormatMenu(
 
     // Hexadecimal action.
     const auto hexAct = addCheckableAction(
-                fmtMenu, tr("Hexadecimal"), on,
+                this, fmtMenu, Tr::tr("Hexadecimal"), on,
                 fmt == PeripheralRegisterFormat::Hexadecimal,
                 [item] {
         item->m_reg.format = PeripheralRegisterFormat::Hexadecimal;
@@ -851,7 +743,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFormatMenu(
 
     // Decimal action.
     const auto decAct = addCheckableAction(
-                fmtMenu, tr("Decimal"), on,
+                this, fmtMenu, Tr::tr("Decimal"), on,
                 fmt == PeripheralRegisterFormat::Decimal,
                 [item] {
         item->m_reg.format = PeripheralRegisterFormat::Decimal;
@@ -861,7 +753,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFormatMenu(
 
     // Octal action.
     const auto octAct = addCheckableAction(
-                fmtMenu, tr("Octal"), on,
+                this, fmtMenu, Tr::tr("Octal"), on,
                 fmt == PeripheralRegisterFormat::Octal,
                 [item] {
         item->m_reg.format = PeripheralRegisterFormat::Octal;
@@ -871,7 +763,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFormatMenu(
 
     // Binary action.
     const auto binAct = addCheckableAction(
-                fmtMenu, tr("Binary"), on,
+                this, fmtMenu, Tr::tr("Binary"), on,
                 fmt == PeripheralRegisterFormat::Binary,
                 [item] {
         item->m_reg.format = PeripheralRegisterFormat::Binary;
@@ -885,7 +777,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFormatMenu(
 QMenu *PeripheralRegisterHandler::createRegisterFieldFormatMenu(
         DebuggerState state, PeripheralRegisterFieldItem *item) const
 {
-    const auto fmtMenu = new QMenu(tr("Format"));
+    const auto fmtMenu = new QMenu(Tr::tr("Format"));
     const auto actionGroup = new QActionGroup(fmtMenu);
 
     const bool on = m_engine->hasCapability(RegisterCapability)
@@ -895,7 +787,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFieldFormatMenu(
 
     // Hexadecimal action.
     const auto hexAct = addCheckableAction(
-                fmtMenu, tr("Hexadecimal"), on,
+                this, fmtMenu, Tr::tr("Hexadecimal"), on,
                 fmt == PeripheralRegisterFormat::Hexadecimal,
                 [item] {
         item->m_fld.format = PeripheralRegisterFormat::Hexadecimal;
@@ -905,7 +797,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFieldFormatMenu(
 
     // Decimal action.
     const auto decAct = addCheckableAction(
-                fmtMenu, tr("Decimal"), on,
+                this, fmtMenu, Tr::tr("Decimal"), on,
                 fmt == PeripheralRegisterFormat::Decimal,
                 [item] {
         item->m_fld.format = PeripheralRegisterFormat::Decimal;
@@ -915,7 +807,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFieldFormatMenu(
 
     // Octal action.
     const auto octAct = addCheckableAction(
-                fmtMenu, tr("Octal"), on,
+                this, fmtMenu, Tr::tr("Octal"), on,
                 fmt == PeripheralRegisterFormat::Octal,
                 [item] {
         item->m_fld.format = PeripheralRegisterFormat::Octal;
@@ -925,7 +817,7 @@ QMenu *PeripheralRegisterHandler::createRegisterFieldFormatMenu(
 
     // Binary action.
     const auto binAct = addCheckableAction(
-                fmtMenu, tr("Binary"), on,
+                this, fmtMenu, Tr::tr("Binary"), on,
                 fmt == PeripheralRegisterFormat::Binary,
                 [item] {
         item->m_fld.format = PeripheralRegisterFormat::Binary;
@@ -936,36 +828,31 @@ QMenu *PeripheralRegisterHandler::createRegisterFieldFormatMenu(
     return fmtMenu;
 }
 
-void PeripheralRegisterHandler::setActiveGroup(bool checked)
+void PeripheralRegisterHandler::setActiveGroup(const QString &groupName)
 {
-    if (!checked)
-        return;
     deactivateGroups();
-    if (const auto act = qobject_cast<QAction *>(sender())) {
-        const QString groupName = act->data().toString();
-        const auto groupEnd = m_peripheralRegisterGroups.end();
-        const auto groupIt = std::find_if(
-                    m_peripheralRegisterGroups.begin(), groupEnd,
-                    [groupName](const PeripheralRegisterGroup &group){
-            return group.name == groupName;
-        });
-        if (groupIt == groupEnd)
-            return; // Group not found.
-        // Set active group.
-        groupIt->active = true;
+    const auto groupEnd = m_peripheralRegisterGroups.end();
+    const auto groupIt = std::find_if(
+                m_peripheralRegisterGroups.begin(), groupEnd,
+                [groupName](const PeripheralRegisterGroup &group){
+        return group.name == groupName;
+    });
+    if (groupIt == groupEnd)
+        return; // Group not found.
+    // Set active group.
+    groupIt->active = true;
 
-        // Add all register items of active register group.
-        m_activeRegisters.reserve(groupIt->registers.count());
-        for (PeripheralRegister &reg : groupIt->registers) {
-            const auto item = new PeripheralRegisterItem(m_engine, *groupIt, reg);
-            rootItem()->appendChild(item);
+    // Add all register items of active register group.
+    m_activeRegisters.reserve(groupIt->registers.count());
+    for (PeripheralRegister &reg : groupIt->registers) {
+        const auto item = new PeripheralRegisterItem(m_engine, *groupIt, reg);
+        rootItem()->appendChild(item);
 
-            const quint64 address = reg.address(groupIt->baseAddress);
-            m_activeRegisters.insert(address, item);
-        }
-
-        m_engine->reloadPeripheralRegisters();
+        const quint64 address = reg.address(groupIt->baseAddress);
+        m_activeRegisters.insert(address, item);
     }
+
+    m_engine->reloadPeripheralRegisters();
 }
 
 void PeripheralRegisterHandler::deactivateGroups()
@@ -978,5 +865,4 @@ void PeripheralRegisterHandler::deactivateGroups()
     m_activeRegisters.clear();
 }
 
-} // namespace Internal
-} // namespace Debugger
+} // Debugger::Internal
