@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "mcusupportoptionspage.h"
 #include "mcukitmanager.h"
@@ -29,6 +7,7 @@
 #include "mcusupportconstants.h"
 #include "mcusupportoptions.h"
 #include "mcusupportsdk.h"
+#include "mcusupporttr.h"
 #include "mcutarget.h"
 #include "settingshandler.h"
 
@@ -49,6 +28,7 @@
 #include <QGroupBox>
 #include <QHBoxLayout>
 #include <QLabel>
+#include <QMessageBox>
 #include <QPushButton>
 #include <QVBoxLayout>
 
@@ -56,14 +36,12 @@ namespace McuSupport::Internal {
 
 class McuSupportOptionsWidget : public Core::IOptionsPageWidget
 {
-    Q_DECLARE_TR_FUNCTIONS(McuSupport::Internal::McuSupportOptionsWidget)
-
 public:
     McuSupportOptionsWidget(McuSupportOptions &, const SettingsHandler::Ptr &);
 
     void updateStatus();
     void showMcuTargetPackages();
-    McuTargetPtr currentMcuTarget() const;
+    [[nodiscard]] McuTargetPtr currentMcuTarget() const;
 
 private:
     void apply() final;
@@ -95,7 +73,7 @@ McuSupportOptionsWidget::McuSupportOptionsWidget(McuSupportOptions &options,
     : m_options{options}
     , m_settingsHandler(settingsHandler)
 {
-    auto mainLayout = new QVBoxLayout(this);
+    auto *mainLayout = new QVBoxLayout(this);
 
     {
         m_statusInfoLabel = new Utils::InfoLabel;
@@ -103,25 +81,28 @@ McuSupportOptionsWidget::McuSupportOptionsWidget(McuSupportOptions &options,
         m_statusInfoLabel->setOpenExternalLinks(false);
         mainLayout->addWidget(m_statusInfoLabel);
         connect(m_statusInfoLabel, &QLabel::linkActivated, this, [] {
-            Core::ICore::showOptionsDialog(CMakeProjectManager::Constants::CMAKE_SETTINGS_PAGE_ID);
+            Core::ICore::showOptionsDialog(CMakeProjectManager::Constants::Settings::TOOLS_ID);
         });
     }
 
     {
-        m_qtForMCUsSdkGroupBox = new QGroupBox(m_options.qtForMCUsSdkPackage->label());
+        m_qtForMCUsSdkGroupBox = new QGroupBox(Tr::tr("Qt for MCUs SDK"));
         m_qtForMCUsSdkGroupBox->setFlat(true);
-        auto layout = new QVBoxLayout(m_qtForMCUsSdkGroupBox);
+        auto *layout = new QVBoxLayout(m_qtForMCUsSdkGroupBox);
+        // Re-read the qtForMCUs package from settings to discard un-applied changes from previous sessions
+        m_options.qtForMCUsSdkPackage->readFromSettings();
+
         layout->addWidget(m_options.qtForMCUsSdkPackage->widget());
         mainLayout->addWidget(m_qtForMCUsSdkGroupBox);
     }
 
     {
         m_mcuTargetsGroupBox = new QGroupBox(
-            tr("Targets supported by the %1").arg(m_qtForMCUsSdkGroupBox->title()));
+            Tr::tr("Targets supported by the %1").arg(m_qtForMCUsSdkGroupBox->title()));
         m_mcuTargetsGroupBox->setFlat(true);
         mainLayout->addWidget(m_mcuTargetsGroupBox);
         m_mcuTargetsComboBox = new QComboBox;
-        auto layout = new QVBoxLayout(m_mcuTargetsGroupBox);
+        auto *layout = new QVBoxLayout(m_mcuTargetsGroupBox);
         layout->addWidget(m_mcuTargetsComboBox);
         connect(m_mcuTargetsComboBox,
                 &QComboBox::currentTextChanged,
@@ -134,7 +115,7 @@ McuSupportOptionsWidget::McuSupportOptionsWidget(McuSupportOptions &options,
     }
 
     {
-        m_packagesGroupBox = new QGroupBox(tr("Requirements"));
+        m_packagesGroupBox = new QGroupBox(Tr::tr("Requirements"));
         m_packagesGroupBox->setFlat(true);
         mainLayout->addWidget(m_packagesGroupBox);
         m_packagesLayout = new QFormLayout;
@@ -148,7 +129,7 @@ McuSupportOptionsWidget::McuSupportOptionsWidget(McuSupportOptions &options,
 
     {
         m_kitAutomaticCreationCheckBox = new QCheckBox(
-            tr("Automatically create kits for all available targets on start"));
+            Tr::tr("Automatically create kits for all available targets on start"));
         connect(m_kitAutomaticCreationCheckBox, &QCheckBox::stateChanged, this, [this](int state) {
             m_options.setAutomaticKitCreationEnabled(state == Qt::CheckState::Checked);
         });
@@ -156,24 +137,24 @@ McuSupportOptionsWidget::McuSupportOptionsWidget(McuSupportOptions &options,
     }
 
     {
-        m_kitCreationGroupBox = new QGroupBox(tr("Create a Kit"));
+        m_kitCreationGroupBox = new QGroupBox(Tr::tr("Create a Kit"));
         m_kitCreationGroupBox->setFlat(true);
         mainLayout->addWidget(m_kitCreationGroupBox);
         m_kitCreationInfoLabel = new Utils::InfoLabel;
-        auto vLayout = new QHBoxLayout(m_kitCreationGroupBox);
+        auto *vLayout = new QHBoxLayout(m_kitCreationGroupBox);
         vLayout->addWidget(m_kitCreationInfoLabel);
-        m_kitCreationPushButton = new QPushButton(tr("Create Kit"));
+        m_kitCreationPushButton = new QPushButton(Tr::tr("Create Kit"));
         m_kitCreationPushButton->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
         connect(m_kitCreationPushButton, &QPushButton::clicked, this, [this] {
             McuKitManager::newKit(currentMcuTarget().get(), m_options.qtForMCUsSdkPackage);
             m_options.registerQchFiles();
             updateStatus();
         });
-        m_kitUpdatePushButton = new QPushButton(tr("Update Kit"));
+        m_kitUpdatePushButton = new QPushButton(Tr::tr("Update Kit"));
         m_kitUpdatePushButton->setSizePolicy(m_kitCreationPushButton->sizePolicy());
         connect(m_kitUpdatePushButton, &QPushButton::clicked, this, [this] {
-            for (auto kit : McuKitManager::upgradeableKits(currentMcuTarget().get(),
-                                                           m_options.qtForMCUsSdkPackage))
+            for (auto *kit : McuKitManager::upgradeableKits(currentMcuTarget().get(),
+                                                            m_options.qtForMCUsSdkPackage))
                 McuKitManager::upgradeKitInPlace(kit,
                                                  currentMcuTarget().get(),
                                                  m_options.qtForMCUsSdkPackage);
@@ -212,11 +193,11 @@ void McuSupportOptionsWidget::updateStatus()
             m_mcuTargetsInfoLabel->setType(Utils::InfoLabel::NotOk);
             const Utils::FilePath sdkPath = m_options.qtForMCUsSdkPackage->basePath();
             QString deprecationMessage;
-            if (Sdk::checkDeprecatedSdkError(sdkPath, deprecationMessage))
+            if (checkDeprecatedSdkError(sdkPath, deprecationMessage))
                 m_mcuTargetsInfoLabel->setText(deprecationMessage);
             else
-                m_mcuTargetsInfoLabel->setText(tr("No valid kit descriptions found at %1.")
-                                                   .arg(Sdk::kitsPath(sdkPath).toUserOutput()));
+                m_mcuTargetsInfoLabel->setText(Tr::tr("No valid kit descriptions found at %1.")
+                                                   .arg(kitsPath(sdkPath).toUserOutput()));
         }
     }
 
@@ -242,13 +223,13 @@ void McuSupportOptionsWidget::updateStatus()
 
             m_kitCreationInfoLabel->setText(
                 hasMatchingKits
-                    ? tr("A kit for the selected target and SDK version already exists.")
-                : hasUpgradeableKits ? tr("Kits for a different SDK version exist.")
-                                     : tr("A kit for the selected target can be created."));
+                    ? Tr::tr("A kit for the selected target and SDK version already exists.")
+                : hasUpgradeableKits ? Tr::tr("Kits for a different SDK version exist.")
+                                     : Tr::tr("A kit for the selected target can be created."));
         } else {
             m_kitCreationInfoLabel->setType(Utils::InfoLabel::NotOk);
-            m_kitCreationInfoLabel->setText("Provide the package paths in order to create a kit "
-                                            "for your target.");
+            m_kitCreationInfoLabel->setText(Tr::tr("Provide the package paths to create a kit "
+                                            "for your target."));
         }
     }
 
@@ -260,12 +241,21 @@ void McuSupportOptionsWidget::updateStatus()
         m_statusInfoLabel->setVisible(!cMakeAvailable);
         if (m_statusInfoLabel->isVisible()) {
             m_statusInfoLabel->setType(Utils::InfoLabel::NotOk);
-            m_statusInfoLabel->setText("No CMake tool was detected. Add a CMake tool in the "
-                                       "<a href=\"cmake\">CMake options</a> and press Apply.");
+            m_statusInfoLabel->setText(Tr::tr("No CMake tool was detected. Add a CMake tool in the "
+                                       "<a href=\"cmake\">CMake options</a> and select Apply."));
         }
     }
 }
 
+struct McuPackageSort {
+    bool operator()(McuPackagePtr a, McuPackagePtr b) const {
+        if (a->cmakeVariableName() != b->cmakeVariableName())
+            return a->cmakeVariableName() > b->cmakeVariableName();
+        else
+            return a->environmentVariableName() > b->environmentVariableName();
+    }
+};
+
 void McuSupportOptionsWidget::showMcuTargetPackages()
 {
     McuTargetPtr mcuTarget = currentMcuTarget();
@@ -273,15 +263,28 @@ void McuSupportOptionsWidget::showMcuTargetPackages()
         return;
 
     while (m_packagesLayout->rowCount() > 0) {
-        QFormLayout::TakeRowResult row = m_packagesLayout->takeRow(0);
-        row.labelItem->widget()->hide();
-        row.fieldItem->widget()->hide();
+        m_packagesLayout->removeRow(0);
     }
 
-    for (const auto &package : qAsConst(m_options.sdkRepository.packages)) {
-        QWidget *packageWidget = package->widget();
-        if (!mcuTarget->packages().contains(package) || package->label().isEmpty())
+    std::set<McuPackagePtr, McuPackageSort> packages;
+
+    for (const auto &package : mcuTarget->packages()) {
+        if (package->label().isEmpty())
             continue;
+        packages.insert(package);
+    }
+
+    for (const auto &package : packages) {
+        QWidget *packageWidget = package->widget();
+        std::weak_ptr packagePtr(package);
+        connect(package.get(), &McuPackage::reset, this, [this, packagePtr] {
+            McuPackagePtr package = packagePtr.lock();
+            if (package) {
+                MacroExpanderPtr macroExpander
+                    = m_options.sdkRepository.getMacroExpander(*currentMcuTarget());
+                package->setPath(macroExpander->expand(package->defaultPath()));
+            }
+        });
         m_packagesLayout->addRow(package->label(), packageWidget);
         packageWidget->show();
     }
@@ -310,8 +313,33 @@ void McuSupportOptionsWidget::apply()
     bool pathsChanged = false;
 
     m_settingsHandler->setAutomaticKitCreation(m_options.automaticKitCreationEnabled());
+    m_options.sdkRepository.expandVariablesAndWildcards();
+
+    if (m_mcuTargetsComboBox->count() == 0)
+        return;
+
+    QMessageBox warningPopup(QMessageBox::Icon::Warning,
+                             Tr::tr("Warning"),
+                             Tr::tr("Cannot apply changes in SDKs > MCU."),
+                             QMessageBox::Ok,
+                             this);
+
+    auto target = currentMcuTarget();
+    if (!target) {
+        warningPopup.setInformativeText(Tr::tr("No target selected."));
+        warningPopup.exec();
+        return;
+    }
+    if (!target->isValid()) {
+        warningPopup.setInformativeText(
+            Tr::tr("Invalid paths present for target\n%1")
+                .arg(McuKitManager::generateKitNameFromTarget(target.get())));
+        warningPopup.exec();
+        return;
+    }
+
     pathsChanged |= m_options.qtForMCUsSdkPackage->writeToSettings();
-    for (auto package : qAsConst(m_options.sdkRepository.packages))
+    for (const auto &package : target->packages())
         pathsChanged |= package->writeToSettings();
 
     if (pathsChanged) {
@@ -324,10 +352,17 @@ void McuSupportOptionsWidget::populateMcuTargetsComboBox()
 {
     m_options.populatePackagesAndTargets();
     m_mcuTargetsComboBox->clear();
+    int initialPlatformIndex = 0;
+    int targetsCounter = -1;
     m_mcuTargetsComboBox->addItems(
-        Utils::transform<QStringList>(m_options.sdkRepository.mcuTargets, [](const McuTargetPtr &t) {
+        Utils::transform<QStringList>(m_options.sdkRepository.mcuTargets, [&](const McuTargetPtr &t) {
+            if (t->platform().name == m_settingsHandler->initialPlatformName())
+                initialPlatformIndex = m_options.sdkRepository.mcuTargets.indexOf(t);
+            targetsCounter++;
             return McuKitManager::generateKitNameFromTarget(t.get());
         }));
+    if (targetsCounter != -1)
+        m_mcuTargetsComboBox->setCurrentIndex(initialPlatformIndex);
     updateStatus();
 }
 
@@ -335,8 +370,8 @@ McuSupportOptionsPage::McuSupportOptionsPage(McuSupportOptions &options,
                                              const SettingsHandler::Ptr &settingsHandler)
 {
     setId(Utils::Id(Constants::SETTINGS_ID));
-    setDisplayName(McuSupportOptionsWidget::tr("MCU"));
-    setCategory(ProjectExplorer::Constants::DEVICE_SETTINGS_CATEGORY);
+    setDisplayName(Tr::tr("MCU"));
+    setCategory(ProjectExplorer::Constants::SDK_SETTINGS_CATEGORY);
     setWidgetCreator([&options, &settingsHandler] {
         return new McuSupportOptionsWidget(options, settingsHandler);
     });
