@@ -39,7 +39,7 @@ void NimTextEditorWidget::findLinkAt(const QTextCursor &c, const Utils::LinkHand
 {
     const Utils::FilePath &path = textDocument()->filePath();
 
-    NimSuggest *suggest = NimSuggestCache::instance().get(path);
+    NimSuggest *suggest = Nim::Suggest::getFromCache(path);
     if (!suggest)
         return processLinkCallback(Utils::Link());
 
@@ -48,7 +48,7 @@ void NimTextEditorWidget::findLinkAt(const QTextCursor &c, const Utils::LinkHand
     int line = 0, column = 0;
     Utils::Text::convertPosition(document(), c.position(), &line, &column);
 
-    std::shared_ptr<NimSuggestClientRequest> request = suggest->def(path.toString(),
+    std::shared_ptr<NimSuggestClientRequest> request = suggest->def(path.toUrlishString(),
                                                                              line,
                                                                              column,
                                                                              dirtyFile->fileName());
