@@ -11,6 +11,8 @@
 #include "tests/qmlpreviewplugin_test.h"
 #endif
 
+#include <android/androidconstants.h>
+
 #include <coreplugin/icore.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
@@ -20,8 +22,8 @@
 
 #include <extensionsystem/pluginmanager.h>
 
+#include <projectexplorer/devicesupport/devicekitaspects.h>
 #include <projectexplorer/kit.h>
-#include <projectexplorer/kitaspects.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorerconstants.h>
@@ -40,10 +42,11 @@
 #include <qtsupport/qtversionmanager.h>
 #include <qtsupport/baseqtversion.h>
 
+#include <solutions/tasking/tasktreerunner.h>
+
 #include <texteditor/texteditor.h>
 
-#include <android/androidconstants.h>
-
+#include <utils/async.h>
 #include <utils/icon.h>
 #include <utils/mimeconstants.h>
 #include <utils/proxyaction.h>
@@ -55,22 +58,10 @@
 #include <QToolBar>
 
 using namespace ProjectExplorer;
+using namespace Tasking;
 
 namespace QmlPreview {
 
-class QmlPreviewParser : public QObject
-{
-    Q_OBJECT
-public:
-    QmlPreviewParser();
-    void parse(const QString &name, const QByteArray &contents,
-               QmlJS::Dialect::Enum dialect);
-
-signals:
-    void success(const QString &changedFile, const QByteArray &contents);
-    void failure();
-};
-
 static QByteArray defaultFileLoader(const QString &filename, bool *success)
 {
     if (Core::DocumentModel::Entry *entry
@@ -103,9 +94,6 @@ static std::unique_ptr<QmlDebugTranslationClient> defaultCreateDebugTranslationC
     return client;
 }
 
-static void defaultRefreshTranslationFunction()
-{}
-
 class QmlPreviewPluginPrivate : public QObject
 {
 public:
@@ -119,25 +107,9 @@ public:
     void checkEditor();
     void checkFile(const QString &fileName);
     void triggerPreview(const QString &changedFile, const QByteArray &contents);
-
-    QString previewedFile() const;
-    void setPreviewedFile(const QString &previewedFile);
-    QmlPreviewRunControlList runningPreviews() const;
-
-    QmlPreviewFileClassifier fileClassifier() const;
-    void setFileClassifier(QmlPreviewFileClassifier fileClassifier);
-
-    float zoomFactor() const;
-    void setZoomFactor(float zoomFactor);
-
-    QmlPreview::QmlPreviewFpsHandler fpsHandler() const;
-    void setFpsHandler(QmlPreview::QmlPreviewFpsHandler fpsHandler);
-
-    QString locale() const;
-    void setLocale(const QString &locale);
+    void checkDocument(const QString &name, const QByteArray &contents, QmlJS::Dialect::Enum dialect);
 
     QmlPreviewPlugin *q = nullptr;
-    QThread m_parseThread;
     QString m_previewedFile;
     QPointer<Core::IEditor> m_lastEditor;
     QmlPreviewRunControlList m_runningPreviews;
@@ -148,6 +120,8 @@ public:
 
     QmlPreviewRunnerSetting m_settings;
     QmlPreviewRunWorkerFactory runWorkerFactory;
+
+    TaskTreeRunner m_parseRunner;
 };
 
 QmlPreviewPluginPrivate::QmlPreviewPluginPrivate(QmlPreviewPlugin *parent)
@@ -157,7 +131,6 @@ QmlPreviewPluginPrivate::QmlPreviewPluginPrivate(QmlPreviewPlugin *parent)
     m_settings.fileClassifier = &defaultFileClassifier;
     m_settings.fpsHandler = &defaultFpsHandler;
     m_settings.createDebugTranslationClientMethod = &defaultCreateDebugTranslationClientMethod;
-    m_settings.refreshTranslationsFunction = &defaultRefreshTranslationFunction;
 
     Core::ActionContainer *menu = Core::ActionManager::actionContainer(
                 Constants::M_BUILDPROJECT);
@@ -171,10 +144,10 @@ QmlPreviewPluginPrivate::QmlPreviewPluginPrivate(QmlPreviewPlugin *parent)
         if (auto multiLanguageAspect = QmlProjectManager::QmlMultiLanguageAspect::current())
             m_localeIsoCode = multiLanguageAspect->currentLocale();
         bool skipDeploy = false;
-        const Kit *kit = ProjectManager::startupTarget()->kit();
-        if (ProjectManager::startupTarget() && kit)
+        if (const Kit *kit = activeKitForActiveProject()) {
             skipDeploy = kit->supportedPlatforms().contains(Android::Constants::ANDROID_DEVICE_TYPE)
-                || DeviceTypeKitAspect::deviceTypeId(kit) == Android::Constants::ANDROID_DEVICE_TYPE;
+                || RunDeviceTypeKitAspect::deviceTypeId(kit) == Android::Constants::ANDROID_DEVICE_TYPE;
+        }
         ProjectExplorerPlugin::runStartupProject(Constants::QML_PREVIEW_RUN_MODE, skipDeploy);
     });
     menu->addAction(
@@ -226,13 +199,7 @@ QmlPreviewPluginPrivate::QmlPreviewPluginPrivate(QmlPreviewPlugin *parent)
         toolBar->insertAction(nullptr, action);
     });
 
-    m_parseThread.start();
-    QmlPreviewParser *parser = new QmlPreviewParser;
-    parser->moveToThread(&m_parseThread);
-    connect(&m_parseThread, &QThread::finished, parser, &QObject::deleteLater);
-    connect(q, &QmlPreviewPlugin::checkDocument, parser, &QmlPreviewParser::parse);
     connect(q, &QmlPreviewPlugin::previewedFileChanged, this, &QmlPreviewPluginPrivate::checkFile);
-    connect(parser, &QmlPreviewParser::success, this, &QmlPreviewPluginPrivate::triggerPreview);
 }
 
 QmlPreviewPlugin::~QmlPreviewPlugin()
@@ -250,13 +217,6 @@ void QmlPreviewPlugin::initialize()
 #endif
 }
 
-ExtensionSystem::IPlugin::ShutdownFlag QmlPreviewPlugin::aboutToShutdown()
-{
-    d->m_parseThread.quit();
-    d->m_parseThread.wait();
-    return SynchronousShutdown;
-}
-
 QString QmlPreviewPlugin::previewedFile() const
 {
     return d->m_previewedFile;
@@ -376,7 +336,7 @@ void QmlPreviewPlugin::previewCurrentFile()
                              Tr::tr("Start the QML Preview for the project before selecting "
                                     "a specific file for preview."));
 
-    const QString file = currentNode->filePath().toString();
+    const QString file = currentNode->filePath().toUrlishString();
     if (file != d->m_previewedFile)
         setPreviewedFile(file);
     else
@@ -421,7 +381,7 @@ void QmlPreviewPluginPrivate::onEditorAboutToClose(Core::IEditor *editor)
 void QmlPreviewPluginPrivate::setDirty()
 {
     m_dirty = true;
-    QTimer::singleShot(1000, this, [&](){
+    QTimer::singleShot(1000, this, [this] {
         if (m_dirty && m_lastEditor) {
             m_dirty = false;
             checkEditor();
@@ -495,7 +455,7 @@ void QmlPreviewPluginPrivate::checkEditor()
         dialect = QmlJS::Dialect::QmlQtQuick2Ui;
     else
         dialect = QmlJS::Dialect::NoLanguage;
-    emit q->checkDocument(doc->filePath().toString(), doc->contents(), dialect);
+    checkDocument(doc->filePath().toUrlishString(), doc->contents(), dialect);
 }
 
 void QmlPreviewPluginPrivate::checkFile(const QString &fileName)
@@ -507,11 +467,8 @@ void QmlPreviewPluginPrivate::checkFile(const QString &fileName)
     const QByteArray contents = m_settings.fileLoader(fileName, &success);
 
     if (success) {
-        emit q->checkDocument(fileName,
-                              contents,
-                              QmlJS::ModelManagerInterface::guessLanguageOfFile(
-                                  Utils::FilePath::fromUserInput(fileName))
-                                  .dialect());
+        checkDocument(fileName, contents, QmlJS::ModelManagerInterface::guessLanguageOfFile(
+                                              Utils::FilePath::fromUserInput(fileName)).dialect());
     }
 }
 
@@ -523,29 +480,32 @@ void QmlPreviewPluginPrivate::triggerPreview(const QString &changedFile, const Q
         emit q->updatePreviews(m_previewedFile, changedFile, contents);
 }
 
-QmlPreviewParser::QmlPreviewParser()
+static void parse(QPromise<void> &promise, const QString &name, const QByteArray &contents,
+                  QmlJS::Dialect::Enum dialect)
 {
-    static const int dialectMeta = qRegisterMetaType<QmlJS::Dialect::Enum>();
-    Q_UNUSED(dialectMeta)
+    if (!QmlJS::Dialect(dialect).isQmlLikeOrJsLanguage())
+        return;
+
+    auto qmljsDoc = QmlJS::Document::create(Utils::FilePath::fromString(name), dialect);
+    if (promise.isCanceled())
+        return;
+
+    qmljsDoc->setSource(QString::fromUtf8(contents));
+    if (!qmljsDoc->parse())
+        promise.future().cancel();
 }
 
-void QmlPreviewParser::parse(const QString &name, const QByteArray &contents,
-                             QmlJS::Dialect::Enum dialect)
+void QmlPreviewPluginPrivate::checkDocument(const QString &name, const QByteArray &contents,
+                                            QmlJS::Dialect::Enum dialect)
 {
-    if (!QmlJS::Dialect(dialect).isQmlLikeOrJsLanguage()) {
-        emit success(name, contents);
-        return;
-    }
-
-    QmlJS::Document::MutablePtr qmljsDoc = QmlJS::Document::create(Utils::FilePath::fromString(name),
-                                                                   dialect);
-    qmljsDoc->setSource(QString::fromUtf8(contents));
-    if (qmljsDoc->parse())
-        emit success(name, contents);
-    else
-        emit failure();
+    const auto onParseSetup = [name, contents, dialect](Utils::Async<void> &async) {
+        async.setConcurrentCallData(parse, name, contents, dialect);
+    };
+    const auto onDone = [this, name, contents](DoneWith result) {
+        if (result == DoneWith::Success)
+            triggerPreview(name, contents);
+    };
+    m_parseRunner.start({Utils::AsyncTask<void>(onParseSetup)}, {}, onDone);
 }
 
 } // namespace QmlPreview
-
-#include <qmlpreviewplugin.moc>
